/*!
*	\addtogroup Modulo3
*	\{
*/
/*!
*	\addtogroup TestModulo3 Specifiche per il Test
*	\{
*/

/*!
*	\brief Test per le funzioni di interfaccia del Modulo 3
*
* Programma di test per il Modulo 3
*
* Ogni test e' identificato da un numero cosi' composto:\n
* \p xyynn \n
*  x	Id del modulo\n
* yy	Id della funzione\n
* nn	Id del test\n
*
* Ogni test ha anche un peso che viene indicato nella stampa dei risultati.
* I test di funzionalit hanno i pesi piu' elevati.
* In questo test viene verificato che l'errore ritornato sia effettivamente 
* quello indicato dalla funzione.
*
* Vengono effettuati i seguenti controlli: 
* - Inserimento (InsertBlankRecord()): 
*   - Handle del DBF errato
*	- Parametro Where errato
*   - Verifica funzionalita' 
*
* - Spostamento (GotoRecord()):
*   - Handle del DBF errato
*   - Parametro Mode errato
*   - Verifica funzionalita'
* 
* - Cancellazione (DeleteRecord()):
*   - Handle del DBF errato
*   - Verifica funzionalit
*   - Cancellazione record non valido
*
* - Misto
*	- Inserimento di piu' record in varie posizioni, spostamenti e cancellazioni
****************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "LPC_Include.h"

static int handle = -1;

void TestInsert(void);
void TestGoto(void);
void TestDelete(void);
void TestMisto(void);
void Result(char *Func);
int ConfrontaFiles(char *File1, char *File2);
int Startup(int handle);
int Poke(char *Nome, int n, char c);

// Vettori per gli id dei test, il check e i pesi relativi
#define MAX_TEST 100
typedef struct {
	int testid;
	int weight;
	char desc[256];
	int passed;
} TEST_t;
		
static int numtest;
static TEST_t test[MAX_TEST];

int main(int argc, char **argv)
{
	char scelta[10];
	int sel;

	printf("Programma di Test per il Modulo 3\n\n");

	// Pulizia iniziale e preparazione dell'handle
	handle = Startup(handle);
	if (handle < 0)
	{
		printf("TEST: Errore apertura database:[%d]\n", handle);
		printf("Exiting....\n");
		return(-1);
	}

	for(sel = 0;;)
	{
		if (argc < 2)
		{
			printf("\t1. Test funzione InsertBlankRecord\n");
			printf("\t2. Test funzione GotoRecord\n");
			printf("\t3. Test funzione DeleteRecord\n");
			printf("\t4. Test funzione Misto\n");
			printf("\t5. Test completo e valutazione\n");
			printf("\n\t0. Uscita\n");
			printf("\n\nScelta: ");

			(void) fgets(scelta, sizeof(scelta), stdin);
			sel = atoi(scelta);
		}
		else // Faccio solo 1 giro
		{
			if (sel)
				sel = 0;
			else
				sel = atoi(argv[1]);
		}

		Result(NULL);
		switch(sel)
		{
		case 0:
			CloseDatabaseFile(handle);
		//	printf("Bye!\n\n");
			return(0);
		case 1:
			TestInsert();
			Result("INSERTBLANK");	
			break;
		case 2:
			TestGoto();
			Result("GOTO");	
			break;
		case 3:
			TestDelete();
			Result("DELETERECORD");	
			break;
		case 4:
			TestMisto();
			Result("MISTO");	
			break;
		case 5:
			// Devo ripulire prima di ricominciare
			handle = Startup(handle);
			if (handle < 0)
			{
				printf("TEST: Errore apertura database:[%d]\n", handle);
				printf("Exiting....\n");
				return(-1);
			}
			TestInsert();
			TestGoto();
			TestDelete();
			TestMisto();
			Result("FINALE");
			break;

		default:
			printf("Scelta errata...\n");
			break;;
		}
	}
}
void TestInsert()
{
	int ret= 0;
	DBF_INFO_t DBFInfo;

	printf("Test INSERT\n");

// Test sull'handle
	test[numtest].testid = 30101;
	test[numtest].weight = 3;
	strcpy(test[numtest].desc, "INSERT: Handle errato");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	ret =  InsertBlankRecord(5, TOP);
	printf("Ritorno della InsertBlankRecord: [%d]\n", ret);
	if (ret == LPC_BAD_ARG)
		test[numtest].passed = 1;
	numtest++;

// Test sul Where
	test[numtest].testid = 30102;
	test[numtest].weight = 3;
	strcpy(test[numtest].desc, "INSERT: Where errato");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	ret =  InsertBlankRecord(handle, 333);
	printf("Ritorno della InsertBlankRecord: [%d]\n", ret);
	if (ret == LPC_BAD_ARG)
		test[numtest].passed = 1;
	numtest++;

// Verifica funzionalita'
	test[numtest].testid = 30103;
	test[numtest].weight = 10;
	strcpy(test[numtest].desc, "INSERT: Funzionalita'(1)");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	for(;;)
	{
		ret = Poke("Prova.dbf", 1, 'A');
		if (ret != 0)
			break;

		ret =  InsertBlankRecord(handle, TOP);
		printf("Ritorno della InsertBlankRecord TOP: [%d]\n", ret);
		if (ret != LPC_OK)
			break;

		ret = GetDBFInfo(handle, &DBFInfo);
		if (ret != LPC_OK)
			break;

		if ((DBFInfo.NumRecs != 2) || (DBFInfo.CurrentRec != 1))
			break;

		if (ConfrontaFiles("Prova.dbf", "ProvaTest2.dbf"))
			break;

		test[numtest].passed = 1;
		break;
	}
	numtest++;

// Verifica funzionalita'
	test[numtest].testid = 30104;
	test[numtest].weight = 10;
	strcpy(test[numtest].desc, "INSERT: Funzionalita'(2)");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	for(;;)
	{
		ret = Poke("Prova.dbf", 1, 'B');
		if (ret != 0)
			break;

		ret =  InsertBlankRecord(handle, BOTTOM);
		printf("Ritorno della InsertBlankRecord BOTTOM: [%d]\n", ret);
		if (ret != LPC_OK)
			break;

		ret = GetDBFInfo(handle, &DBFInfo);
		if (ret != LPC_OK)
			break;

		if ((DBFInfo.NumRecs != 3) || (DBFInfo.CurrentRec != 3))
			break;

		if (ConfrontaFiles("Prova.dbf", "ProvaTest3.dbf"))
			break;

		test[numtest].passed = 1;
		break;
	}	
	numtest++;

// Verifica funzionalita'
	test[numtest].testid = 30105;
	test[numtest].weight = 10;
	strcpy(test[numtest].desc, "INSERT: Funzionalita'(3)");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	for(;;)
	{
		ret = Poke("Prova.dbf", 3, 'C');
		if (ret != 0)
			break;

		ret =  InsertBlankRecord(handle, CURRENT);
		printf("Ritorno della InsertBlankRecord CURRENT: [%d]\n", ret);
		if (ret != LPC_OK)
			break;

		ret = GetDBFInfo(handle, &DBFInfo);
		if (ret != LPC_OK)
			break;

		if ((DBFInfo.NumRecs != 4) || (DBFInfo.CurrentRec != 3))
			break;

		if (ConfrontaFiles("Prova.dbf", "ProvaTest4.dbf"))
			break;

		test[numtest].passed = 1;
		break;
	}	
	numtest++;

	// SE TUTTO OK IL DBF CONTIENE 4 RECORD E IL RECORD CORRENTE E' IL 3
}

void TestGoto()
{
	int ret= 0;
	DBF_INFO_t DBFInfo;

	printf("Test GOTO\n");

// Test sull'handle
	test[numtest].testid = 30201;
	test[numtest].weight = 3;
	strcpy(test[numtest].desc, "GOTO: Handle errato");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	ret =  GotoRecord(-3, ABSOLUTE, 2);
	printf("Ritorno della GotoRecord: [%d]\n", ret);
	if (ret == LPC_BAD_ARG)
		test[numtest].passed = 1;
	numtest++;

// Test Mode errato
	test[numtest].testid = 30202;
	test[numtest].weight = 3;
	strcpy(test[numtest].desc, "GOTO: Mode errato");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	ret =  GotoRecord(handle, 99, 2);
	printf("Ritorno della GotoRecord: [%d]\n", ret);
	if (ret == LPC_BAD_ARG)
		test[numtest].passed = 1;
	numtest++;

// Verifica funzionalita'
	test[numtest].testid = 30203;
	test[numtest].weight = 5;
	strcpy(test[numtest].desc, "GOTO: Funzionalita' (ABSOLUTE:1)");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	ret =  GotoRecord(handle, ABSOLUTE, 2);
	printf("Ritorno della GotoRecord: [%d]\n", ret);
	// Il while serve per i break
	while(ret == 2)
	{
		ret = GetDBFInfo(handle, &DBFInfo);
		if (ret != LPC_OK)
			break;

		if (DBFInfo.CurrentRec != 2)
			break;

		test[numtest].passed = 1;
		break;
	}	
	numtest++;
 
	test[numtest].testid = 30204;
	test[numtest].weight = 5;
	strcpy(test[numtest].desc, "GOTO: Funzionalita' (ABSOLUTE:2)");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	ret =  GotoRecord(handle, ABSOLUTE, 100);
	printf("Ritorno della GotoRecord: [%d]\n", ret);
	// Il while serve per i break
	while(ret == LPC_ERR_EOF)
	{
		ret = GetDBFInfo(handle, &DBFInfo);
		if (ret != LPC_OK)
			break;

		if (DBFInfo.CurrentRec != 5)
			break;

		test[numtest].passed = 1;
		break;
	}	
	numtest++;

	test[numtest].testid = 30205;
	test[numtest].weight = 5;
	strcpy(test[numtest].desc, "GOTO: Funzionalita' (ABSOLUTE:3)");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	ret =  GotoRecord(handle, ABSOLUTE, -1);
	printf("Ritorno della GotoRecord: [%d]\n", ret);
	// Il while serve per i break
	while(ret == 1)
	{
		ret = GetDBFInfo(handle, &DBFInfo);
		if (ret != LPC_OK)
			break;

		if (DBFInfo.CurrentRec != 1)
			break;

		test[numtest].passed = 1;
		break;
	}	
	numtest++;

	test[numtest].testid = 30206;
	test[numtest].weight = 5;
	strcpy(test[numtest].desc, "GOTO: Funzionalita' (ABSOLUTE:4)");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	ret =  GotoRecord(handle, ABSOLUTE, -1);
	printf("Ritorno della GotoRecord: [%d]\n", ret);
	// Il while serve per i break
	while(ret == 1)
	{
		ret = GetDBFInfo(handle, &DBFInfo);
		if (ret != LPC_OK)
			break;

		if (DBFInfo.CurrentRec != 1)
			break;

		test[numtest].passed = 1;
		break;
	}	
	numtest++;

	test[numtest].testid = 30207;
	test[numtest].weight = 5;
	strcpy(test[numtest].desc, "GOTO: Funzionalita' (RELATIVE:1)");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	ret =  GotoRecord(handle, RELATIVE, 1);
	printf("Ritorno della GotoRecord: [%d]\n", ret);
	// Il while serve per i break
	while(ret == 2)
	{
		ret = GetDBFInfo(handle, &DBFInfo);
		if (ret != LPC_OK)
			break;

		if (DBFInfo.CurrentRec != 2)
			break;

		test[numtest].passed = 1;
		break;
	}	
	numtest++;

	test[numtest].testid = 30208;
	test[numtest].weight = 5;
	strcpy(test[numtest].desc, "GOTO: Funzionalita' (RELATIVE:2)");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	ret =  GotoRecord(handle, RELATIVE, 100);
	printf("Ritorno della GotoRecord: [%d]\n", ret);
	while(ret == LPC_ERR_EOF)
	{
		ret = GetDBFInfo(handle, &DBFInfo);
		if (ret != LPC_OK)
			break;

		if (DBFInfo.CurrentRec != 5)
			break;

		test[numtest].passed = 1;
		break;
	}	
	numtest++;

	test[numtest].testid = 30209;
	test[numtest].weight = 5;
	strcpy(test[numtest].desc, "GOTO: Funzionalita' (RELATIVE:3)");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	ret =  GotoRecord(handle, RELATIVE, -1);
	printf("Ritorno della GotoRecord: [%d]\n", ret);
	while(ret == 4)
	{
		ret = GetDBFInfo(handle, &DBFInfo);
		if (ret != LPC_OK)
			break;

		if (DBFInfo.CurrentRec != 4)
			break;

		test[numtest].passed = 1;
		break;
	}	
	numtest++;

	test[numtest].testid = 30210;
	test[numtest].weight = 5;
	strcpy(test[numtest].desc, "GOTO: Funzionalita' (RELATIVE:3)");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	ret =  GotoRecord(handle, RELATIVE, 0);
	printf("Ritorno della GotoRecord: [%d]\n", ret);
	while(ret == 4)
	{
		ret = GetDBFInfo(handle, &DBFInfo);
		if (ret != LPC_OK)
			break;

		if (DBFInfo.CurrentRec != 4)
			break;

		test[numtest].passed = 1;
		break;
	}	
	numtest++;

	// Se tutto ok i record sono 4 e il corrente  il 4
}


void TestDelete()
{
	int ret = 0;
	DBF_INFO_t DBFInfo;

	printf("Test DELETE\n");

// Test sull'handle
	test[numtest].testid = 30301;
	test[numtest].weight = 3;
	strcpy(test[numtest].desc, "DELETE: Handle errato");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	ret =  DeleteRecord(100);
	printf("Ritorno della DeleteRecord: [%d]\n", ret);
	if (ret == LPC_BAD_ARG)
		test[numtest].passed = 1;
	numtest++;

// Test funzionalita'
	test[numtest].testid = 30302;
	test[numtest].weight = 10;
	strcpy(test[numtest].desc, "DELETE: Funzionalita' (1)");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	ret =  DeleteRecord(handle);
	printf("Ritorno della DeleteRecord: [%d]\n", ret);
	while(ret == LPC_OK)
	{
		ret = GetDBFInfo(handle, &DBFInfo);
		if (ret != LPC_OK)
			break;

		if ((DBFInfo.NumRecs != 3) || (DBFInfo.CurrentRec != 4))
			break;

		if (ConfrontaFiles("Prova.dbf", "ProvaTest3.dbf"))
			break;

		test[numtest].passed = 1;
		break;
	}	
	numtest++;

	test[numtest].testid = 30303;
	test[numtest].weight = 10;
	strcpy(test[numtest].desc, "DELETE: Funzionalita' (2)");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	ret = GotoRecord(handle, ABSOLUTE, 1000);
	if (ret == LPC_ERR_EOF)
	{
		ret =  DeleteRecord(handle);
		printf("Ritorno della DeleteRecord: [%d]\n", ret);
		if (ret == LPC_ERR_EOF)
			test[numtest].passed = 1;
	}
	numtest++;

	// Se tutto OK i record sono 3 e il corrente e' il 3
}



void TestMisto()
{
	int ret = 0, i;
	DBF_INFO_t DBFInfo;

	printf("Test MISTO\n");

	test[numtest].testid = 30401;
	test[numtest].weight = 15;
	strcpy(test[numtest].desc, "MISTO: Funzionalita' miste");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);

	// Il for mi serve solo come struttura per evitare il GOTO
	for(;;)
	{
		// Prendo il numero di rec, torno al primo record, li cancello tutti, ne inserisco 2 correnti
		// poi ne cancello 1 poi ne inserisco uno alla fine poi uno al top poi verifico che i 
		// record siano 3 e che sono al primo
		ret = GetDBFInfo(handle, &DBFInfo);
		printf("Ritorno della GetDBFInfo: [%d]\n", ret);
		if (ret != LPC_OK)
			break;
		ret = GotoRecord(handle, ABSOLUTE, 0);
		printf("Ritorno della GotoRecord: [%d]\n", ret);
		if (ret != 1)
			break;
		for (i = 0; i < DBFInfo.NumRecs; i++)
		{
			ret = DeleteRecord(handle);
			printf("Ritorno della DeleteRecord: [%d]\n", ret);
			if (ret != LPC_OK)
				break;
		}
		if (ret != LPC_OK)
			break;
		// Provo a cancellarne uno in piu'
		ret = DeleteRecord(handle);
		printf("Ritorno della DeleteRecord: [%d]\n", ret);
		if (ret != LPC_ERR_EOF)
			break;

		// 1
		ret = InsertBlankRecord(handle, CURRENT);
		printf("Ritorno della InsertBlankRecord: [%d]\n", ret);
		if (ret != LPC_OK)
			break;
		if (Poke("Prova.dbf", 1, 'A') != 0)
			break;

		// 2
		ret = InsertBlankRecord(handle, CURRENT);
		printf("Ritorno della InsertBlankRecord: [%d]\n", ret);
		if (ret != LPC_OK)
			break;

		// 1
		ret = DeleteRecord(handle);
		printf("Ritorno della DeleteRecord: [%d]\n", ret);
		if (ret != LPC_OK)
			break;

		// 2
		ret = InsertBlankRecord(handle, BOTTOM);
		printf("Ritorno della InsertBlankRecord: [%d]\n", ret);
		if (ret != LPC_OK)
			break;
		if (Poke("Prova.dbf", 2, 'B') != 0)
			break;

		// 3
		ret = InsertBlankRecord(handle, TOP);
		printf("Ritorno della InsertBlankRecord: [%d]\n", ret);
		if (ret != LPC_OK)
			break;
		if (Poke("Prova.dbf", 1, 'C') != 0)
			break;

		ret = GetDBFInfo(handle, &DBFInfo);
		printf("Ritorno della GetDBFInfo: [%d]\n", ret);
		if (ret != LPC_OK)
			break;

		// Verifico che i record siano 3 e che il corrente sia il primo
		// poi comunque ho finito
		if ((DBFInfo.NumRecs != 3) || (DBFInfo.CurrentRec != 1))
		{
			printf("Numero record o record corrente errato\n", ret);
			break;
		}
		if (ConfrontaFiles("Prova.dbf", "ProvaTest5.dbf"))
			break;

		test[numtest].passed = 1;
		break;
	}
	numtest++;
}

/*********************************
* RESULT
* Stampa i risultati. Se il parametro e' NULL allora resetta i valori
**********************************/
void Result(char *Func)
{
	int i, quanti, totval;

	// Svuoto il vettore
	if (!Func)
	{
		for (i = 0; i < MAX_TEST; i++)
			memset(&test[i], 0, sizeof(TEST_t));
		numtest = 0;
		return;
	}

	// Stampo gli ID
	printf("\n");
	printf("ID:   ");
	for (i = 0; i < numtest; i++)
	{
		printf("%6d", test[i].testid);
	}

	// Stampo i pesi e prendo il valore totale
	printf("\n");
	printf("PESO: ");
	totval = 0;
	for (i = 0; i < numtest; i++)
	{
		printf("%6d", test[i].weight);
		totval += test[i].weight;
	}

	// Stampo i risultati
	printf("\n");
	printf("PASS: ");
	quanti = 0;
	for (i = 0; i < numtest; i++)
	{
		printf("%6d", test[i].passed);
		if (test[i].passed)
			quanti += test[i].weight;
	}

	printf("\n\nFine Test [%s]: Numtests:[%d] Totvalue:[%d] Passed:[%d] Score:[%3.2f]\n\n", Func, numtest, totval, quanti, (double) (quanti*100/((double)totval)));
}


#define FILELEN 1024
int ConfrontaFiles(char *File1, char *File2)
{
	int len, len1, len2;
	char buf1[FILELEN], buf2[FILELEN];
	FILE *fp1, *fp2;

	fp1 = fopen(File1, "r");
	len1 = fread(buf1, sizeof(char), FILELEN, fp1);
	fclose(fp1);

	fp2 = fopen(File2, "r");
	len2 = fread(buf2, sizeof(char), FILELEN, fp2);
	fclose(fp2);
	if (len1 != len2)
		return(-1);

	// Azzero il timestamp
	len = DBF_NAME_LENGTH+1+sizeof(short)+sizeof(long)+sizeof(short)+sizeof(short);
	memset(buf1+len, 0, sizeof(long));
	memset(buf2+len, 0, sizeof(long));

	// Confronto i buffer
	if (memcmp(buf1, buf2, len1))
		return(-1);

	return 0;
}

int Startup(int handle)
{
	if (IsDBFHandle(handle) == LPC_OK)
		CloseDatabaseFile(handle);

	// Cancello, ricopio dal template e apro il DBF
	remove("Prova.dbf");
	remove("Prova.$$$");
	system("cp ProvaTest1.dbf Prova.dbf");
	return(OpenDatabaseFile("Prova"));
}

#define RECORDLEN 38
#define HEADERLEN 117
int Poke(char *Nome, int n, char c)
{
	long pos;
	FILE *fp1;

	fp1 = fopen(Nome, "rb+");
	if (!fp1)
	{
		printf("ERRORE FOPEN: [%d][%s]\n", errno, strerror(errno));
		return(-99);
	}

	pos = HEADERLEN + (RECORDLEN * (n-1)) + 4;

	if (fseek(fp1, pos, SEEK_SET) == -1)
	{
		printf("ERRORE FSEEK: [%d][%s]\n", errno, strerror(errno));
		fclose(fp1);
		return(-99);
	}
	if (fputc(c, fp1) != c)
	{
		printf("ERRORE FPUTC: [%d][%s]\n", errno, strerror(errno));
		fclose(fp1);
		return(-99);
	}

	fclose(fp1);

	return 0;
}

/*! \} */	// Fine gruppo TestModulo3
/*! \} */	// Fine gruppo Modulo3



